% Written by Anne Gehre

%% conjugate gradients
function C = conjgrad(ATAs, As, Bs,b, ATs, BTs, BBTs, ETDs, C, n_constraintsSQ, n_constraintsLin, idx)

    r = ETDs - A(idx, ATAs, As, Bs, ATs, BTs, BBTs, C);
    p = r;
    rsold = sum(sum(r .* r));

    n_iter = size(C,1)*( n_constraintsSQ + n_constraintsLin);
    for i =1:n_iter
        
        Ap =A(idx, ATAs, As, Bs, ATs, BTs, BBTs, p);
        alpha = rsold / sum(sum(p .* Ap));
        C = C + alpha * p;
        r = r - alpha * Ap;
        rsnew = sum(sum(r.* r));
        %rsnew
        if sqrt(rsnew) < 1e-10
            break;
        end
        p = r + (rsnew / rsold) * p;
        rsold = rsnew;
    end

end

%% evaluate objective function
function [f,g]  = objective(C)
 global As_ Bs_ b_ ATAs_ BBTs_
 C_ =  reshape(C, [120,120]);
 Cb = b_*C_;
 XX =  cellfun(@(x1,y1) (x1*C_-(Cb)*y1).^2, As_, Bs_, 'UniformOutput',false);
 f = 0.5*(sum(sum(sum(cat(3,XX{:}),3))));
 g = vec(A(ATAs_, As_, Bs_, BBTs_, C_));
end

%% Linear part of gradient
function lhs = A(idx, ATAs, As, B, AT, BT, BBT, C)
    l1 = ATAs * C;
    ATC = AT*C;
    AC = As*C;
    l2 = ATC(idx)*B;
    l3 = AC(idx)*BT;
    %l2 = cellfun(@(x1,y1) -x1'*C*y1-x1*C*y1', As, B, 'UniformOutput',false);
    l4 = C * BBT;
    lhs = gather(l1 + l4 - l2-l3);%sum(cat(3,l2{:}),3);
end


function X = addCells(l2,l3)
   X = cellfun(@(l22,l33)  - l22 - l33 , l2,l3, 'UniformOutput',false) ;
end

% A cell of matrices, C matrix
function X = multCell(A,C)
    X=cellfun(@(x) x*C, A,'un',0);
end

function X = multCell2(C,A)
    X=cellfun(@(x) C*x, A,'un',0);
end
%A and B cells of matrices
function X = multCells(A,B, C)
    X = cellfun(@(x,y) x*C*y, A,B, 'UniformOutput',false);
end
function X = multCells2(A1,B1, A2, B2, C)
    %X = cellfun(@(x1,y1,x2,y2) -x1*C*y1-x2*C*y2, A1,B1,A2,B2, 'UniformOutput',false);
    X = zeros(size(C));
    for i=1:length(A1)
        X = X -A1*C*B1-B2*C*A2;
    end
end

function B = getBlockDiag(C, ncopies)
    [nx, ny] = size(C);  
    B = sparse(nx*ncopies, ny*ncopies);
    for i=1:ncopies
        B((i-1)*nx+1:i*nx, (i-1)*ny+1:i*ny)=C;
    end
end
